@@ -1,5 +1,7 @@
 from aiotfm.errors import PacketError, PacketTooLarge, XXTEAInvalidPacket, XXTEAInvalidKeys
 
+import struct
+
 class Packet:
 	def __init__(self, buffer=None):
 		if buffer is None:
@@ -48,26 +50,23 @@ class Packet:
 	def read8(self):
 		"""Read a single byte from the buffer."""
 		self.pos += 1
-		try:
-			return self.buffer[self.pos-1]
-		except:
-			return 0
+		return self.buffer[self.pos-1]
 
 	def read16(self):
 		"""Read a short (two bytes) from the buffer"""
-		return (self.read8() << 8) | self.read8()
+		return struct.unpack('>H', self.readBytes(2))[0]
 
 	def read24(self):
 		"""Read three bytes from the buffer"""
-		return (self.read16() << 8) | self.read8()
+		return int.from_bytes(self.readBytes(3), 'big')
 
 	def read32(self):
 		"""Read an int (four bytes) from the buffer"""
-		return (self.read24() << 8) | self.read8()
+		return struct.unpack('>I', self.readBytes(4))[0]
 
 	def readBool(self):
 		"""Read a boolean (one byte) from the buffer"""
-		return self.read8()==1
+		return self.read8() == 1
 
 	def readString(self):
 		"""return a encoded string (in bytes)"""
@@ -91,20 +90,23 @@ class Packet:
 
 	def write8(self, value):
 		"""Write a single byte to the buffer"""
-		self.buffer.append(value&0xff)
+		self.buffer.append(value & 0xff)
 		return self
 
 	def write16(self, value):
 		"""Write a short (two bytes) to the buffer"""
-		return self.write8(value>>8).write8(value)
+		self.buffer.extend(struct.pack('>H', value & 0xffff))
+		return self
 
 	def write24(self, value):
 		"""Write three bytes to the buffer"""
-		return self.write16(value>>8).write8(value)
+		self.buffer.extend((value & 0xffffff).to_bytes(3, 'big'))
+		return self
 
 	def write32(self, value):
 		"""Write an int (four bytes) to the buffer"""
-		return self.write24(value>>8).write8(value)
+		self.buffer.extend(struct.pack('>I', value & 0xffffffff))
+		return self
 
 	def writeBool(self, value):
 		"""Write a boolean (one byte) to the buffer"""
@@ -157,20 +159,19 @@ class Packet:
 			raise XXTEAInvalidPacket("The Packet is empty.")
 		if len(key)<4:
 			raise XXTEAInvalidKeys(str(key))
-		while len(self.buffer)<10:
-			self.write8(0)
 
-		chunks = []
 		ccc = self.read16()
 		length = len(self.buffer)-2
-		for i in range(length//4+(length%4>0)):
-			chunks.append(self.read32())
+		if length % 4 > 0:
+			pad = 4 - length % 4
+			self.buffer.extend(bytes(pad))
+			length += pad
 
-		chunks = xxtea_encode(chunks, len(chunks), key)
+		chunks = struct.unpack(f'>{length//4}I', self.readBytes(length))
+		chunks = xxtea_encode(list(chunks), len(chunks), key)
 
 		packet = Packet.new(ccc).write16(len(chunks))
-		for chunk in chunks:
-			packet.write32(chunk)
+		packet.writeBytes(struct.pack(f'>{len(chunks)}I', *chunks))
 
 		self.buffer = packet.buffer
 		return self
