@@ -2,8 +2,8 @@ from datetime import datetime
 
 class Date(datetime):
 	@classmethod
-	def fromtimestamp(cls, timestamp, **kw):
-		return super().fromtimestamp(timestamp*60, **kw)
+	def fromtimestamp(cls, timestamp):
+		return cls(*super().fromtimestamp(timestamp*60).timetuple()[:6])
 
 	def timestamp(self):
 		return round(super().timestamp()/60)
